import matplotlib.pyplot as plt
import numpy as np


class Board:
    def __init__(self, cell, blank):
        self.cell = cell
        self.blank = blank
        self.height, self.width = self.cell.shape

    def draw_board(self, ax):
        ax.axis("off")
        ax.set(aspect="equal", xlim=(0, self.width), ylim=(0, self.height))

        ax = self._draw_inner_lines(ax, self.cell)
        ax = self._draw_outer_lines(ax, self.cell, color="k", ls="-", lw=4, zorder=4)
        ax = self._draw_empty_cells(ax, self.cell, color="#f5efe6")
        return ax

    def _draw_inner_lines(self, ax, cell):
        left = (self.cell[:, :-1] != self.blank)
        right = (self.cell[:, 1:] != self.blank)
        top = (self.cell[:-1, :] != self.blank)
        bottom = (self.cell[1:, :] != self.blank)
        # thin
        thin_vline_x = np.where(left * right)[1] + 1
        thin_vline_y = self.height - 1 - np.where(left * right)[0]
        thin_hline_x = np.where(top * bottom)[1]
        thin_hline_y = self.height - 1 - np.where(top * bottom)[0]
        ax.plot([thin_vline_x, thin_vline_x], [thin_vline_y, thin_vline_y + 1], color="#EBEBEB", ls="-", lw=1)
        ax.plot([thin_hline_x, thin_hline_x + 1], [thin_hline_y, thin_hline_y], color="#EBEBEB", ls="-", lw=1)
        # bold
        draw_bold_vline = np.logical_or(~left * right, left * ~right)
        bold_vline_x = np.where(draw_bold_vline)[1] + 1
        bold_vline_y = self.height - 1 - np.where(draw_bold_vline)[0]
        draw_bold_hline = np.logical_or(~top * bottom, top * ~bottom)
        bold_hline_x = np.where(draw_bold_hline)[1]
        bold_hline_y = self.height - 1 - np.where(draw_bold_hline)[0]
        ax.plot([bold_vline_x, bold_vline_x], [bold_vline_y, bold_vline_y + 1], color="k", ls="-", lw=1, zorder=4)
        ax.plot([bold_hline_x, bold_hline_x + 1], [bold_hline_y, bold_hline_y], color="k", ls="-", lw=1, zorder=4)
        return ax

    def _draw_outer_lines(self, ax, cell, **kwargs):
        ax.plot([0, 0, self.width, self.width, 0], [0, self.height, self.height, 0, 0], **kwargs)
        return ax

    def _draw_empty_cells(self, ax, cell, color="#f5efe6"):
        cmap = plt.cm.viridis
        cmap.set_over(color, alpha=1)
        cmap.set_under("white", alpha=0)
        ax.imshow(self.cell == self.blank, extent=[0, self.width, 0, self.height], cmap=cmap, vmin=0.49, vmax=0.51)
        return ax

    def draw_answer(self, ax, cell, **kwargs):
        for i in range(self.height):
            for j in range(self.width):
                x = j + 0.5
                y = self.height - i - 0.6
                rotation = 0
                # The rotation process for vertical long tones
                if self.cell[i, j] == "ー" and j >= 1 and self.cell[i, j - 1] == self.blank:
                    x += 0.01
                    y += 0.15
                    rotation = 90
                kwargs["rotation"] = rotation
                ax.text(x, y, self.cell[i, j], **kwargs)
        return ax



from PuzzleImage.Board import Board
from PuzzleImage.PuzzleImage import PuzzleImage
from PuzzleImage.WordList import WordList


class SkeltonImage(PuzzleImage):
    def __init__(self, blank="", *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.blank = blank

    def get_board(self, ax, cell, title=None, w_count=None, is_answer=False):
        """
        Create a puzzle board axes.
        
        Parameters
        ----------
        ax : matplotlib.pyplot.axes
            Axes to be plotted.
        cell : numpy ndarray
            Puzzle board.
        title : str, default ""
            Puzzle name.
        is_answer : bool, default False
            If True, draw with the answer.
        """
        board = Board(cell, self.blank)
        ax = board.draw_board(ax)
        if title:
            ax = self._draw_title(ax, title, x=0.1, y=board.height + 0.2, size=16, ha="left", color="#1a1a1a")
        if w_count:
            ax = self._draw_title(ax, f"{w_count}語", x=board.width, y=board.height + 0.1, size=12, ha="right",
                                  color="#1a1a1a")
        if is_answer:
            ax = board.draw_answer(ax, cell, size=18, ha="center", va="center")
        return ax

    def _draw_title(slef, ax, title, x, y, **kwargs):
        ax.text(x, y, str(title), **kwargs)
        return ax

    def _draw_word_count(slef, ax, count, x, y, **kwargs):
        ax.text(x, y, str(count), **kwargs)
        return ax

    def get_wordlist(self, ax, words, width, height, size):
        wl = WordList(words, width, height, size)
        ax = wl.draw_wordlist(ax)
        return ax

    def get_width(self, words, width, height, size):
        wl = WordList(words, width, height, size)
        widthinch = wl.cal_width()
        return widthinch



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

from setuptools import find_packages
from distutils.core import setup

try:
    with open('README.md') as f:
        readme = f.read()
except IOError:
    readme = ''


def _requires_from_file(filename):
    return open(filename).read().splitlines()


extensions = []
setup(
    name="puzzleimage",
    version="0.0.7",
    url='https://github.com/MakePuzz/puzzleimage',
    author='The puzzle-japan Team',
    author_email='puzzle.hokkaido@gmail.com',
    maintainer='MakePuzz',
    maintainer_email='puzzle.hokkaido@gmail.com',
    description='A Python library to make puzzle images',
    long_description=readme,
    packages=find_packages(),
    ext_modules=extensions,
    install_requires=_requires_from_file('requirements.txt'),
    python_requires='>=3.6',
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
    ]
)



import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import numpy as np

class WordList:
    def __init__(self, words, width, height, fontsize):

        self.width = width
        self.height = height
        self.fontsize = fontsize
        self.inch_of = lambda size: fontsize / 72
        self.words = np.array(sorted(words, key=lambda word: (len(word), word)))
        self.w_num = len(self.words)
        self.row_num = int(height / self.inch_of(fontsize)) - 8
        self.char_max_per_row = int(height / self.inch_of(fontsize)) - 1
        self.col_num = int(np.ceil(self.w_num / self.row_num))
        if self.w_num != 0:
            self.w_lens = np.vectorize(len)(self.words)
            # redefine row number
            self.row_num = int(np.ceil(self.w_num / self.col_num))  # row number


    def draw_wordlist(self, ax):
       
        ax.set(xlim=(0, self.width), ylim=(0, self.height))
        ax.axis("off")

        if self.w_num == 0:
            ax.text(self.width, 0, '© MakePuzz', size=self.fontsize, ha='right', va='bottom', fontname='Yu Gothic', alpha=0.5, fontweight='bold')
            exit()
            
	    # write list
        box = {
            "fc": "#f5efe6", # facecolor
            "ec": "darkgray", # edgecolor
            "style": mpatches.BoxStyle("Round", pad=0.05*self.fontsize/30),
            "size": 0.15 * self.fontsize / 30,
            "difx": 0.3 * self.fontsize / 30, # difference from word_x to draw box
            "dify": 0.25 * self.fontsize / 30, # difference from word_y to draw box
        }
        label = {
            "difx": 0.55* self.fontsize / 30, # difference from word_x to draw the label
            "dify": 0.2 * self.fontsize / 30, # difference from word_y to draw the label
            "size": 14 * self.fontsize / 30, # label font size
            "color": "dimgray",
        }
        labelline = {
            "difx": 0.45 * self.fontsize / 30, # difference from word_x to draw the label line
            "width": 3 * self.fontsize / 30, # line width
            # "space": self.char_max_per_row / (int(self.height / self.inch_of(self.fontsize)) - 6),
            "space": 1.3,
            "ymin_dif": self.inch_of(self.fontsize) * 0.8 , # coefficient of ymin when drawing a label
            "ymax_dif": 0.05 * self.fontsize / 30 # coefficient of ymax when drawing a label
        }
        

        # k: array number of words
        # j: column number
        # i: row number
        k = 0
        word_x = self.inch_of(self.fontsize) * 2
        ymax_default = (self.height * 0.995 - labelline["ymax_dif"]) / self.height
        for j in range(self.col_num):
            if j > 0:
                word_x += (self.w_lens[self.row_num * j] + 2) * self.inch_of(self.fontsize)
            ymax = ymax_default
            for i in range(self.row_num):
                if k == self.w_num:
                    break
                # box
                word_y = self.height * 0.995 - i * self.inch_of(self.fontsize) * labelline["space"]
                fancybox = mpatches.FancyBboxPatch((word_x-box["difx"], word_y-box["dify"]), box["size"], box["size"], boxstyle=box["style"], fc=box["fc"], ec=box["ec"], alpha=1)
                ax.add_patch(fancybox)
                # main word
                ax.text(word_x, word_y, self.words[k], size=self.fontsize, ha="left", va="top")

                # label
                if k == 0 or self.w_lens[k] > self.w_lens[k-1]:
                    ax.text(word_x-label["difx"], word_y-label["dify"], str(self.w_lens[k]), fontsize=label["size"], color=label["color"], ha="right")
                # label line
                if i != 0 and self.w_lens[k] > self.w_lens[k-1]:
                    ymin = (self.height * 0.995 - (i-1) * self.inch_of(self.fontsize) * labelline["space"] - labelline["ymin_dif"]) / self.height
                    ax.axvline(x=word_x-labelline["difx"], color="lightgray", ymin=ymin, ymax=ymax, lw=labelline["width"])
                    ymax = (word_y - labelline["ymax_dif"]) / self.height
                k += 1
            if j == self.col_num-1 and k == self.w_num and self.w_num%self.row_num != 0 :
                ymin = (self.height*0.995 - self.inch_of(self.fontsize) * (i-1) * labelline["space"] - labelline["ymin_dif"]) / self.height
            else:
                ymin = (self.height*0.995 - self.inch_of(self.fontsize) * (i) * labelline["space"] - labelline["ymin_dif"]) / self.height
            ax.axvline(x=word_x-labelline["difx"], color="lightgray", ymin=ymin, ymax=ymax, lw=labelline["width"])
            
            # puzzle copyright
        ax.text(self.width, 0, '© MakePuzz', size=self.fontsize, ha='right', va='bottom', fontname='Yu Gothic', alpha=0.5, fontweight='bold')
        
        return ax

    def cal_width(self):

        word_x = self.inch_of(self.fontsize) * 2
        for j in range(self.col_num):
            if j > 0:
                word_x += (self.w_lens[self.row_num * j] + 2) * self.inch_of(self.fontsize)
        word_x += (self.w_lens[self.w_num - 1]) * self.inch_of(self.fontsize)
                
        return word_x
